@@ -20,10 +20,6 @@
 #define _my_thread_var loc_thread_var
 
 #include <my_config.h>
-#include <stdio.h>
-#include <time.h>
-#include <string.h>
-#include <fcntl.h>
 #include <assert.h>
 
 #ifndef _WIN32
@@ -76,6 +72,7 @@ static void closelog() {}
 */
 
 #if !defined(MYSQL_DYNAMIC_PLUGIN) && !defined(MARIADB_ONLY)
+#include <typelib.h>
 #define MARIADB_ONLY
 #endif /*MYSQL_PLUGIN_DYNAMIC*/
 
@@ -83,11 +80,12 @@ static void closelog() {}
 #define MYSQL_SERVICE_LOGGER_INCLUDED
 #endif /*MARIADB_ONLY*/
 
+#include <my_global.h>
 #include <my_base.h>
-//#include <my_dir.h>
 #include <typelib.h>
 #include <mysql/plugin.h>
 #include <mysql/plugin_audit.h>
+#include <string.h>
 #ifndef RTLD_DEFAULT
 #define RTLD_DEFAULT NULL
 #endif
@@ -441,6 +439,7 @@ static const char *syslog_facility_names[]=
   "LOG_LOCAL4", "LOG_LOCAL5", "LOG_LOCAL6", "LOG_LOCAL7",
   0
 };
+#ifndef _WIN32
 static unsigned int syslog_facility_codes[]=
 {
   LOG_USER, LOG_MAIL, LOG_DAEMON, LOG_AUTH,
@@ -455,6 +454,7 @@ static unsigned int syslog_facility_codes[]=
   LOG_LOCAL0, LOG_LOCAL1, LOG_LOCAL2, LOG_LOCAL3,
   LOG_LOCAL4, LOG_LOCAL5, LOG_LOCAL6, LOG_LOCAL7,
 };
+#endif
 static TYPELIB syslog_facility_typelib=
 {
     array_elements(syslog_facility_names) - 1, "syslog_facility_typelib",
@@ -472,11 +472,13 @@ static const char *syslog_priority_names[]=
   0
 };
 
+#ifndef _WIN32
 static unsigned int syslog_priority_codes[]=
 {
   LOG_EMERG, LOG_ALERT, LOG_CRIT, LOG_ERR,
   LOG_WARNING, LOG_NOTICE, LOG_INFO, LOG_DEBUG,
 };
+#endif
 
 static TYPELIB syslog_priority_typelib=
 {
@@ -731,7 +733,7 @@ static int user_coll_fill(struct user_coll *c, char *users,
         internal_stop_logging= 1;
         CLIENT_ERROR(1, "User '%.*s' was removed from the"
             " server_audit_excl_users.",
-            MYF(ME_JUST_WARNING), (int) cmp_length, users);
+            MYF(ME_WARNING), (int) cmp_length, users);
         internal_stop_logging= 0;
         blank_user(cmp_user);
         refill_cmp_coll= 1;
@@ -740,7 +742,7 @@ static int user_coll_fill(struct user_coll *c, char *users,
       {
         internal_stop_logging= 1;
         CLIENT_ERROR(1, "User '%.*s' is in the server_audit_incl_users, "
-            "so wasn't added.", MYF(ME_JUST_WARNING), (int) cmp_length, users);
+            "so wasn't added.", MYF(ME_WARNING), (int) cmp_length, users);
         internal_stop_logging= 0;
         remove_user(users);
         continue;
@@ -1048,7 +1050,7 @@ static int start_logging()
                   "Could not create file '%s'.", alt_fname);
       is_active= 0;
       CLIENT_ERROR(1, "SERVER AUDIT plugin can't create file '%s'.",
-          MYF(ME_JUST_WARNING), alt_fname);
+          MYF(ME_WARNING), alt_fname);
       return 1;
     }
     error_header();
@@ -1107,7 +1109,7 @@ static void setup_connection_connect(struct connection_info *cn,
   cn->log_always= 0;
   cn->thread_id= event->thread_id;
   get_str_n(cn->db, &cn->db_length, sizeof(cn->db),
-            event->database, event->database_length);
+            event->database.str, event->database.length);
   get_str_n(cn->user, &cn->user_length, sizeof(cn->db),
             event->user, event->user_length);
   get_str_n(cn->host, &cn->host_length, sizeof(cn->host),
@@ -1195,7 +1197,7 @@ static void setup_connection_table(struct connection_info *cn,
   cn->log_always= 0;
   cn->query_length= 0;
   get_str_n(cn->db, &cn->db_length, sizeof(cn->db),
-            event->database, event->database_length);
+            event->database.str, event->database.length);
   get_str_n(cn->user, &cn->user_length, sizeof(cn->db),
             event->user, SAFE_STRLEN(event->user));
   get_str_n(cn->host, &cn->host_length, sizeof(cn->host),
@@ -1343,7 +1345,7 @@ static int log_connection_event(const struct mysql_event_connection *event,
                     event->ip, event->ip_length,
                     event->thread_id, 0, type);
   csize+= my_snprintf(message+csize, sizeof(message) - 1 - csize,
-    ",%.*s,,%d", event->database_length, event->database, event->status);
+    ",%.*s,,%d", event->database.length, event->database.str, event->status);
   message[csize]= '\n';
   return write_log(message, csize + 1);
 }
@@ -1766,8 +1768,8 @@ static int log_table(const struct connection_info *cn,
                     event->ip, (unsigned int)SAFE_STRLEN(event->ip),
                     event->thread_id, cn->query_id, type);
   csize+= my_snprintf(message+csize, sizeof(message) - 1 - csize,
-            ",%.*s,%.*s,",event->database_length, event->database,
-                          event->table_length, event->table);
+            ",%.*s,%.*s,",event->database.length, event->database.str,
+                          event->table.length, event->table.str);
   message[csize]= '\n';
   return write_log(message, csize + 1);
 }
@@ -1788,10 +1790,10 @@ static int log_rename(const struct connection_info *cn,
                     event->ip, (unsigned int)SAFE_STRLEN(event->ip),
                     event->thread_id, cn->query_id, "RENAME");
   csize+= my_snprintf(message+csize, sizeof(message) - 1 - csize,
-            ",%.*s,%.*s|%.*s.%.*s,",event->database_length, event->database,
-                         event->table_length, event->table,
-                         event->new_database_length, event->new_database,
-                         event->new_table_length, event->new_table);
+            ",%.*s,%.*s|%.*s.%.*s,",event->database.length, event->database.str,
+                         event->table.length, event->table.str,
+                         event->new_database.length, event->new_database.str,
+                         event->new_table.length, event->new_table.str);
   message[csize]= '\n';
   return write_log(message, csize + 1);
 }
@@ -1856,7 +1858,7 @@ static void update_connection_info(struct connection_info *cn,
             /* Change DB */
             if (mysql_57_started)
               get_str_n(cn->db, &cn->db_length, sizeof(cn->db),
-                  event->database, event->database_length);
+                  event->database.str, event->database.length);
             else
               get_str_n(cn->db, &cn->db_length, sizeof(cn->db),
                   event->general_query, event->general_query_length);
@@ -1882,9 +1884,9 @@ static void update_connection_info(struct connection_info *cn,
           if (ci_needs_setup(cn))
             setup_connection_query(cn, event);
 
-          if (mode == 0 && cn->db_length == 0 && event->database_length > 0)
+          if (mode == 0 && cn->db_length == 0 && event->database.length > 0)
             get_str_n(cn->db, &cn->db_length, sizeof(cn->db),
-                      event->database, event->database_length);
+                      event->database.str, event->database.length);
 
           if (event->general_error_code == 0)
           {
@@ -1899,7 +1901,7 @@ static void update_connection_info(struct connection_info *cn,
                     event->general_query + 4, event->general_query_length - 4);
               else
                 get_str_n(cn->db, &cn->db_length, sizeof(cn->db),
-                    event->database, event->database_length);
+                    event->database.str, event->database.length);
             }
           }
           update_general_user(cn, event);
@@ -1940,9 +1942,9 @@ static void update_connection_info(struct connection_info *cn,
                 event->ip, SAFE_STRLEN(event->ip));
     }
 
-    if (cn->db_length == 0 && event->database_length != 0)
+    if (cn->db_length == 0 && event->database.length != 0)
       get_str_n(cn->db, &cn->db_length, sizeof(cn->db),
-                event->database, event->database_length);
+                event->database.str, event->database.length);
 
     if (mode == 0)
       cn->query_id= event->query_id;
@@ -2130,6 +2132,7 @@ struct mysql_event_general_v8
 
 static void auditing_v8(MYSQL_THD thd, struct mysql_event_general_v8 *ev_v8)
 {
+#ifdef __linux__
 #ifdef DBUG_OFF
   #ifdef __x86_64__
   static const int cmd_off= 4200;
@@ -2151,7 +2154,7 @@ static void auditing_v8(MYSQL_THD thd, struct mysql_event_general_v8 *ev_v8)
   static const int db_len_off= 68;
   #endif /*x86_64*/
 #endif /*DBUG_OFF*/
-
+#endif /* __linux__ */
   struct mysql_event_general event;
 
   if (ev_v8->event_class != MYSQL_AUDIT_GENERAL_CLASS)
@@ -2169,8 +2172,8 @@ static void auditing_v8(MYSQL_THD thd, struct mysql_event_general_v8 *ev_v8)
   event.general_charset= ev_v8->general_charset;
   event.general_time= ev_v8->general_time;
   event.general_rows= ev_v8->general_rows;
-  event.database= 0;
-  event.database_length= 0;
+  event.database.str= 0;
+  event.database.length= 0;
 
   if (event.general_query_length > 0)
   {
@@ -2178,8 +2181,8 @@ static void auditing_v8(MYSQL_THD thd, struct mysql_event_general_v8 *ev_v8)
     event.general_command= "Query";
     event.general_command_length= 5;
 #ifdef __linux__
-    event.database= *(char **) (((char *) thd) + db_off);
-    event.database_length= *(size_t *) (((char *) thd) + db_len_off);
+    event.database.str= *(char **) (((char *) thd) + db_off);
+    event.database.length= *(size_t *) (((char *) thd) + db_len_off);
 #endif /*__linux*/
   }
 #ifdef __linux__
@@ -2591,7 +2594,7 @@ static void update_file_path(MYSQL_THD thd,
       {
         error_header();
         fprintf(stderr, "Logging was disabled..\n");
-        CLIENT_ERROR(1, "Logging was disabled.", MYF(ME_JUST_WARNING));
+        CLIENT_ERROR(1, "Logging was disabled.", MYF(ME_WARNING));
       }
       goto exit_func;
     }
@@ -2761,7 +2764,7 @@ static void update_logging(MYSQL_THD thd,
     start_logging();
     if (!logging)
     {
-      CLIENT_ERROR(1, "Logging was disabled.", MYF(ME_JUST_WARNING));
+      CLIENT_ERROR(1, "Logging was disabled.", MYF(ME_WARNING));
     }
   }
   else
